﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Drawing;
using System.Threading;

namespace CustomerApp1
{
    public class Display
    {
        private SerialPort mySP;
        private GTTResponseProcessor _proc;
        private GTTResponse _CurrentResponse = null;
        private ManualResetEvent _PacketReceiveEvt = new ManualResetEvent(false);
        private int _WaitForResponse = -1;
        public int myPort = 40;

        public int Width = 480;
        public int Height = 272;

        public Display(int port)
        {
            myPort = port;
            
        }
        public bool InitializeDisplay()
        {
            _proc = new GTTResponseProcessor();
            _proc.OnMessageReceived += new GttResponseReceivedDelegate(_proc_OnMessageReceived);
            mySP = new SerialPort("COM" + myPort.ToString(), 115200, Parity.None, 8, StopBits.One);
            try
            {
                mySP.Encoding = Encoding.ASCII;
                mySP.Handshake = Handshake.RequestToSend;
                mySP.Open();
                mySP.ReceivedBytesThreshold = 1;
                mySP.DataReceived += new SerialDataReceivedEventHandler(mySP_DataReceived);
                //mySP.DataReceived += port_OnReceiveDatazz;
                return true;
            }
            catch
            {
                return false;
            }
        }

        GTTResponse SendSyncMessage(byte[] data)
        {
            if (data.Length < 2)
                throw new ArgumentException();
            _WaitForResponse = data[1];
            _PacketReceiveEvt.Reset();
            Write(data);
            _PacketReceiveEvt.WaitOne();
            return _CurrentResponse;
        }

        void _proc_OnMessageReceived(GTTResponse message)
        {
            if (message.CommandID == _WaitForResponse)
            {
                _CurrentResponse = message;
                _PacketReceiveEvt.Set();
            }
        }

        void mySP_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            byte[] dat = new byte[mySP.BytesToRead];
            if (mySP.Read(dat, 0, dat.Length) != dat.Length)
            {
                //This should never happen..
            }
            else
            {
                _proc.Process(dat);
            }
        }


        public void ChangeDrawingColor(Color C)
        {
            byte[] b = new byte[5];
            b[0] = 254;
            b[1] = 99;
            b[2] = C.R;
            b[3] = C.G;
            b[4] = C.B;
            mySP.Write(b, 0, b.Length);
        }
        public void Create9SliceBarGraph(int index, int min, int max, int x, int y, int Width, int Height, int fg, int bg, int orientation)
        {
            CmdBuilder cb = new CmdBuilder(254, 104, (byte)index);
            cb.PushInt16(min);
            cb.PushInt16(max);
            cb.PushInt16(x);
            cb.PushInt16(y);
            cb.PushInt16(Width);
            cb.PushInt16(Height);
            cb.Push((byte)fg);
            cb.Push((byte)bg);
            cb.Push((byte)orientation);
            Write(cb.Data);
        }

        public void CreateBarGraph(int Index, int Min, int Max, int X, int Y, int Width, int Height, Color Foreground, Color Background, int Orientation)
        {
            byte[] b = new byte[22];
            b[0] = 254;
            b[1] = 103;
            b[2] = (byte)Index;
            b[3] = BitConverter.GetBytes((short)Min)[1];
            b[4] = BitConverter.GetBytes((short)Min)[0];
            b[5] = BitConverter.GetBytes((short)Max)[1];
            b[6] = BitConverter.GetBytes((short)Max)[0];
            b[7] = BitConverter.GetBytes((short)X)[1];
            b[8] = BitConverter.GetBytes((short)X)[0];
            b[9] = BitConverter.GetBytes((short)Y)[1];
            b[10] = BitConverter.GetBytes((short)Y)[0];
            b[11] = BitConverter.GetBytes((short)Width)[1];
            b[12] = BitConverter.GetBytes((short)Width)[0];
            b[13] = BitConverter.GetBytes((short)Height)[1];
            b[14] = BitConverter.GetBytes((short)Height)[0];
            b[15] = Foreground.R;
            b[16] = Foreground.G;
            b[17] = Foreground.B;
            b[18] = Background.R;
            b[19] = Background.G;
            b[20] = Background.B;
            b[21] = (byte)Orientation;
            mySP.Write(b, 0, b.Length);
        }
        public void UpdateSingleBargraph(int Index, int Value)
        {
            byte[] b = new byte[5];
            b[0] = 254;
            b[1] = 105;
            b[2] = (byte)(Index);
            b[3] = BitConverter.GetBytes((short)Value)[1];
            b[4] = BitConverter.GetBytes((short)Value)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void UpdateMultipleBargraphs(int StartIndex, int[] Values)
        {
            byte[] b = new byte[4 + (Values.Length * 2)];
            b[0] = 254;
            b[1] = 106;
            b[2] = (byte)(StartIndex);
            b[3] = (byte)(Values.Length);
            int i = 4;
            foreach (int Value in Values)
            {
                b[i++] = BitConverter.GetBytes((short)Value)[1];
                b[i++] = BitConverter.GetBytes((short)Value)[0];
            }
            mySP.Write(b, 0, b.Length);
        }


        internal void WriteText(string p)
        {
            Write(System.Text.Encoding.ASCII.GetBytes(p));
        }

        internal void DrawLine(Point point, Point point_2)
        {
            CmdBuilder cb = new CmdBuilder(254, 108);
            cb.PushPoint(point);
            cb.PushPoint(point_2);
            Write(cb.Data);
        }

        internal void SetTextInsertionPoint(Point point)
        {
            CmdBuilder cb = new CmdBuilder(254, 121);
            cb.PushPoint(point);
            Write(cb.Data);            
        }

        internal void SetTextColor(Color color)
        {
            byte [] Cmd = {254, 46,color.R,color.G,color.B};
            Write(Cmd);            
        }

        internal void SetScrollMode(byte mode)
        {
            Write(254, 81, mode);
        }

        internal void ChangeTouchReportingStyle(byte style)
        {
            Write(254, 135, style);
        }

        internal void ClearScreen()
        {
            Write(254, 88);
        }
        internal void Write(params byte[] data)
        {
            mySP.Write(data, 0, data.Length);
        }

        internal void EnableManualUpdate(bool p)
        {
                Write(254, 90, (byte) (p ? 1 : 0));
        }

        internal void SetTextWindow(Point point, int w, int h)
        {
            CmdBuilder cb = new CmdBuilder(254, 43);
            cb.PushPoint(point);
            cb.PushInt16(w);
            cb.PushInt16(h);
            Write(cb.Data);
        }
        internal void Load9Slice(int index,string filename)
        {
            CmdBuilder cmd = new CmdBuilder(254, 144, (byte)index);
            cmd.PushString(filename);
            Write(cmd.Data);
        }

        internal void Draw9Slice(int idx, int x, int y, int w, int h)
        {
            CmdBuilder cmd = new CmdBuilder(254, 145, (byte)idx);
            cmd.PushInt16(x);
            cmd.PushInt16(y);
            cmd.PushInt16(w);
            cmd.PushInt16(h);
            Write(cmd.Data);
        }

        internal void ManualUpdate()
        {
            Write(254, 91);
        }

        internal void ClosePort()
        {
            mySP.Close(); 
        }

        internal int _SyncCounter =0;
        internal void Sync()
        {
            CmdBuilder cb = new CmdBuilder(254, 255);
            string msg = String.Format("{0}",_SyncCounter++);
            cb.PushString(msg);
            GTTResponse resp =  SendSyncMessage(cb.Data);
            if (resp.AsString != msg)
            {
                throw new Exception("DeSync!");
            }
        }
    }
}
