﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CustomerApp1
{
    enum ProcessorState
    {
        Prefix = 1,
        CmdID = 2,
        Len1 = 3,
        Len2 = 4,
        Data = 5
    }

    public class GTTResponse
    {
        public byte   CommandID;
        public byte[] Data;

        public string AsString
        {
            get
            {
                return System.Text.Encoding.ASCII.GetString(Data).Replace("\0","");
            }
        }
    }

    public delegate void GttResponseReceivedDelegate (GTTResponse message);

    public class GTTResponseProcessor
    {
        private ProcessorState _State = ProcessorState.Prefix;
        private int _CommandID = 0;
        private int _Len1 =0;
        private int _Len2=1 ;
        private byte[] _Data = null;
        private int _TotalLen = 0;
        private int _BytesLeft = 0;
        
        public event GttResponseReceivedDelegate OnMessageReceived;

        private void ProcessByte(byte b)
        {
            switch (_State)
            {
                case ProcessorState.Prefix: 
                    if (b == 252)
                        _State = ProcessorState.CmdID;
                    else
                    {
                        //Unexpected data, should not happen..
                    }
                    break;
                case ProcessorState.CmdID:  
                    _CommandID = b;
                    _State = ProcessorState.Len1;
                    break;
                case ProcessorState.Len1:
                    _Len1 = b;
                    _State = ProcessorState.Len2;
                    break;
                case ProcessorState.Len2:
                    _Len2 = b;
                    _TotalLen = (_Len1 << 8) | _Len2;
                    if (_TotalLen == 0)
                    {
                        MakeResponse();
                        _State = ProcessorState.Prefix;
                    }
                    else
                    {
                        _Data = new byte[_TotalLen];
                        _BytesLeft = _TotalLen;
                        _State = ProcessorState.Data;
                             
                    }
                    break;
                case ProcessorState.Data:
                    _Data[_TotalLen - _BytesLeft] = b;
                    _BytesLeft--;
                    if (_BytesLeft == 0)
                    {
                        MakeResponse();
                        _State = ProcessorState.Prefix;
                    }
                    break;
            }
        }

        private void MakeResponse()
        {
            GttResponseReceivedDelegate handler = OnMessageReceived;
            if (handler != null)
            {
                GTTResponse resp = new GTTResponse();
                resp.CommandID = (Byte)_CommandID;
                resp.Data = _Data;
                handler(resp);
            }
            _Data = null;
        }

        public void Process(params byte [] data)
        {
            foreach (byte b in data)
                ProcessByte(b);
        }
    }
}
